import streamlit as st
from ucimlrepo import fetch_ucirepo 
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import utility

@st.cache_data(show_spinner=False)
def load_data():
    # fetch dataset 
    car_evaluation = fetch_ucirepo(id=19) 
    
    # data (as pandas dataframes) 
    data = car_evaluation.data.features 
    target = car_evaluation.data.targets 
    return pd.concat([data, target],axis=1)

def define_data_type_and_unique_values(df):
    label=['class']
    ordinal_features = list(set(df.columns) - (set(label)))
    unique_dic = utility.df_to_dict(df)
    label_dic = {'class': ['unacc', 'acc', 'good', 'vgood']}
    ordinal_dic = {k: unique_dic[k] for k in set(unique_dic).intersection(ordinal_features)}
    ordinal_dic['maint'] = np.flip(ordinal_dic['maint'])
    ordinal_dic['buying'] = np.flip(ordinal_dic['buying'])
    return {'feature': ordinal_dic, 'label': label_dic}

def plot_feature_distributions_againts(df, feature, label='class'):
    unique_val_feature = feature_dic['feature'][feature]
    row1 = st.columns([2,2])
    row2 = st.columns([2,2])
    rows = row1+row2
    for i, val in enumerate(unique_val_feature):
        rows[i].bar_chart(df, x = label, y = val, y_label=val)
        
def feature_distributions(df, feature):
    feat_dist = df[feature].value_counts()
    total = feat_dist.sum()
    row1 = st.columns([2,2])
    row2 = st.columns([2,2])
    rows = row1+row2
    for i, feat in enumerate(feat_dist.index):
        rows[i].progress(int((feat_dist[feat]/total)*100), text=feat)

        
def feature_distributions_againts(df, feature_dic, feature, label='class'):
    data_type = 'label' if label == 'class' else 'feature'
    unique_val_feature = feature_dic['feature'][feature]
    count_per_class = {key: [] for key in unique_val_feature}
    for val in unique_val_feature:
        curr_value_count = df[df[feature]==val][label].value_counts()
        for key in feature_dic[data_type][label]:
            count_per_class[val].append(curr_value_count[key] if key in curr_value_count else 0)
    count_per_class.update({label: feature_dic[data_type][label]})
    df_dist = pd.DataFrame(count_per_class)
    plot_feature_distributions_againts(df_dist, feature, label)

def class_distributions(df, feature_dic, label='class'):
    labels = feature_dic['label'][label]
    sizes = [len(df[df['class'] == val]) for val in labels]
    fig = plt.figure(figsize=(5,5))
    plt.pie(sizes, labels=labels, autopct='%1.1f%%')
    st.pyplot(fig)


def show_feature_distributions(df, feature_dic):
    with st.container():
        with st.form(key='feat_dist_form'):
            feature = st.selectbox('select feature', features)
            show_dist = st.form_submit_button('select')
        if show_dist: 
            feature_distributions(df, feature)
        if st.checkbox('show distribution againts classes'):
            feature_distributions_againts(df, feature_dic, feature)
        st.subheader('Feature Distribution againts other Feature')
        with st.form(key='feat_rel_form'):
            feat_2 = st.selectbox('select feature', [f for f in features if f != feature])
            show_rel = st.form_submit_button('select')
        if show_rel: 
            feature_distributions_againts(df, feature_dic, feature, feat_2)


st.title('Data Analysis')
with st.spinner('loading data...'):
    df = load_data()
    feature_dic = define_data_type_and_unique_values(df)
    features = list(feature_dic['feature'].keys())
if st.checkbox('show database'):
    st.dataframe(df)
    
st.header('Distribution of class')
class_distributions(df, feature_dic)

st.header('Feature Distribution')
show_feature_distributions(df, feature_dic)

        
    

